@@ -149,6 +149,8 @@ BindState = SimpleNamespace(
 
 DEV_MODE = __dev_mode__
 
+OTB_MODE = False
+
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
@@ -230,14 +232,14 @@ class DeviceBase(Entity):
     def _make_cmd(self, code, payload="00", **kwargs) -> None:
         super()._make_cmd(code, self.id, payload, **kwargs)
 
-    def _set_ctl(self, ctl) -> None:  # self._ctl
+    def _set_ctl(self, ctl):  # self._ctl
         """Set the device's parent controller, after validating it."""
 
         if self._ctl is ctl:
-            return
+            return self._ctl
         if self._is_controller and not isinstance(self, UfhController):
             # HACK: UFC is/binds to a contlr
-            return
+            return  # TODO
         if self._ctl is not None:
             raise CorruptStateError(f"{self} changed controller: {self._ctl} to {ctl}")
 
@@ -249,8 +251,8 @@ class DeviceBase(Entity):
         ctl.device_by_id[self.id] = self
         ctl.devices.append(self)
 
-        _LOGGER.debug("%s: controller now set to %s", self, ctl)
-        return ctl
+        _LOGGER.debug("%s: controller now set to %s", self, self._ctl)
+        return self._ctl
 
     def _handle_msg(self, msg) -> None:
         assert msg.src is self, f"msg inappropriately routed to {self}"
@@ -308,8 +310,8 @@ class DeviceBase(Entity):
             self._evo = create_system(self._gwy, self, **kwargs)
 
     @property
-    def schema(self) -> dict:
-        """Return the fixed attributes of the device (e.g. TODO)."""
+    def traits(self) -> dict:
+        """Return the traits of the (known) device."""
 
         return {
             **(self._codes if DEV_MODE else {}),
@@ -318,6 +320,11 @@ class DeviceBase(Entity):
             SZ_CLASS: self._klass,
         }
 
+    @property
+    def schema(self):
+        """Return the fixed attributes of the device."""
+        return {}
+
     @property
     def params(self):
         """Return the configurable attributes of the device."""
@@ -347,11 +354,11 @@ class DeviceInfo:  # 10E0
         return self._msg_value(_10E0)
 
     @property
-    def schema(self) -> dict:
-        result = super().schema
-        result.update({self.RF_BIND: self._msg_value(_1FC9)})
+    def traits(self) -> dict:
+        result = super().traits
+        result.update({f"_{self.RF_BIND}": self._msg_value(_1FC9)})
         if _10E0 in self._msgs or _10E0 in RAMSES_DEVICES.get(self._klass, []):
-            result.update({self.DEVICE_INFO: self.device_info})
+            result.update({f"_{self.DEVICE_INFO}": self.device_info})
         return result
 
 
@@ -385,7 +392,7 @@ class Device(DeviceInfo, DeviceBase):
             # TODO: is buggy - remove? how?
             self._set_parent(self._ctl._evo._get_zone(msg.payload["zone_idx"]))
 
-    def _set_parent(self, parent, domain=None) -> None:  # self._parent
+    def _set_parent(self, parent, domain=None):
         """Set the device's parent zone, after validating it.
 
         There are three possible sources for the parent zone of a device:
@@ -426,13 +433,13 @@ class Device(DeviceInfo, DeviceBase):
         self._parent = parent
         self._domain_id = domain
 
-        if hasattr(parent, "devices") and self not in parent.devices:
+        if hasattr(self._parent, "devices") and self not in self._parent.devices:
             parent.devices.append(self)
             parent.device_by_id[self.id] = self
 
         if DEV_MODE:
-            _LOGGER.debug("Device %s: parent now set to %s", self, parent)
-        return parent
+            _LOGGER.debug("Device %s: parent now set to %s", self, self._parent)
+        return self._parent
 
     @property
     def zone(self) -> Optional[Entity]:  # should be: Optional[Zone]
@@ -463,30 +470,36 @@ class Actuator:  # 3EF0, 3EF1
             self._make_cmd(_3EF1, priority=Priority.LOW, retries=1)
 
     @property
-    def _ch_active(self) -> Optional[bool]:  # 3EF0
+    def bit_3_7(self) -> Optional[bool]:  # 3EF0 (byte 3, only OTB)
+        return self._msg_flag(_3EF0, "_flags_3", 7)
+
+    @property
+    def bit_6_6(self) -> Optional[bool]:  # 3EF0 ?dhw_enabled (byte 3, only R8820A?)
+        return self._msg_flag(_3EF0, "_flags_3", 6)
+
+    @property
+    def ch_active(self) -> Optional[bool]:  # 3EF0 (byte 3, only R8820A?)
         return self._msg_value(_3EF0, key="ch_active")
 
     @property
-    def _ch_enabled(self) -> Optional[bool]:  # 3EF0
+    def ch_enabled(self) -> Optional[bool]:  # 3EF0 (byte 6, only R8820A?)
         return self._msg_value(_3EF0, key="ch_enabled")
 
     @property
-    def _dhw_active(self) -> Optional[bool]:  # 3EF0
+    def dhw_active(self) -> Optional[bool]:  # 3EF0 (byte 3, only OTB)
         return self._msg_value(_3EF0, key="dhw_active")
 
     @property
-    def _flame_active(self) -> Optional[bool]:  # 3EF0
+    def flame_active(self) -> Optional[bool]:  # 3EF0 (byte 3, only OTB)
         return self._msg_value(_3EF0, key="flame_active")
 
     @property
-    def _bit_3_7(self) -> Optional[bool]:  # 3EF0
-        if flags := self._msg_value(_3EF0, key="_flags_3"):
-            return flags[7]
+    def ch_setpoint(self) -> Optional[float]:  # 3EF0 (byte 7, only R8820A?)
+        return self._msg_value(_3EF0, key="ch_setpoint")
 
     @property
-    def _bit_6_6(self) -> Optional[bool]:  # 3EF0 ?dhw_enabled
-        if flags := self._msg_value(_3EF0, key="_flags_6"):
-            return flags[6]
+    def max_rel_modulation(self) -> Optional[float]:  # 3EF0 (byte 8, only R8820A?)
+        return self._msg_value(_3EF0, key="max_rel_modulation")
 
     @property
     def actuator_cycle(self) -> Optional[dict]:  # 3EF1
@@ -679,9 +692,9 @@ class Fakeable:
         self._send_cmd(cmd)
 
     @property
-    def schema(self) -> dict:
+    def traits(self) -> dict:
         return {
-            **super().schema,
+            **super().traits,
             SZ_FAKED: self._faked,
         }
 
@@ -1078,11 +1091,11 @@ class UfhController(Device):  # UFC (02):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self._circuits = dict.fromkeys(
-            (f"{i:02X}" for i in range(8)), {"enabled": True}
-        )
+        self._circuits = {}
         self._setpoints = None
         self._heat_demand = None
+        self._relay_demand = None
+        self._relay_demand_fa = None
 
         self._iz_controller = True
 
@@ -1094,7 +1107,7 @@ class UfhController(Device):  # UFC (02):
             self._discover, discover_flag=Discover.SCHEMA, delay=0, period=3600 * 24
         )
         self._gwy._add_task(
-            self._discover, discover_flag=Discover.PARAMS, delay=delay, period=3600
+            self._discover, discover_flag=Discover.PARAMS, delay=delay, period=600
         )
         self._gwy._add_task(
             self._discover, discover_flag=Discover.STATUS, delay=delay + 1, period=60
@@ -1103,50 +1116,70 @@ class UfhController(Device):  # UFC (02):
     @discover_decorator
     def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
+        # Only RPs are: 0001, 0005/000C, 10E0, 000A/2309 & 22D0
 
         if discover_flag & Discover.SCHEMA:
             self._make_cmd(_0005, payload=f"00{_0005_ZONE.UFH}")
-
-        if discover_flag & Discover.PARAMS:
-            [  # only 2309 has any potential?
-                self._make_cmd(_2309, payload=f"{ufh_idx:02X}")
-                for ufh_idx in self._circuits
-                if ufh_idx["enabled"]
+            [
+                self._make_cmd(_000C, payload=f"{ufh_idx:02X}{_000C_DEVICE.UFH}")
+                for ufh_idx in range(8)
             ]
 
+        if discover_flag & Discover.PARAMS:  # only 2309 has any potential?
+            [self._make_cmd(_000A, payload=ufh_idx) for ufh_idx in self.circuits]
+            [self._make_cmd(_2309, payload=ufh_idx) for ufh_idx in self.circuits]
+
+        # if discover_flag & Discover.STATUS:  # only 2309 has any potential?
+        #     [self._make_cmd(_2309, payload=ufh_idx)for ufh_idx in self._circuits_alt]
+
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
-        if msg.code == _0005:
-            # _000C_DEVICE.UFH doesn't seem to work with all UFCs??
-            for ufh_idx, flag in enumerate(msg.payload["zone_mask"]):
-                self._circuits[ufh_idx]["enabled"] = bool(flag)
+        if msg.code == _0005:  # system_zones
+            if msg.payload["_device_class"] not in ("00", "04", "09"):
+                return  # ALL, SENsor, UFH
 
-            # This is done in the zone's discover
-            [
-                self._make_cmd(_000C, payload=f"{ufh_idx:02X}{_000C_DEVICE.UFH}")
-                for ufh_idx in self._circuits
-                if ufh_idx["enabled"]
-            ]
+            for idx, flag in enumerate(msg.payload["zone_mask"]):
+                ufh_idx = f"{idx:02X}"
+                if not flag:
+                    self._circuits.pop(ufh_idx, None)
+                elif "zone_idx" not in self._circuits.get(ufh_idx, {}):
+                    self._circuits[ufh_idx] = {"zone_idx": None}
+                    self._make_cmd(_000C, payload=f"{ufh_idx}{_000C_DEVICE.UFH}")
 
-        elif msg.code == _000C:
-            if msg.payload["zone_id"] is not None:
-                self._circuits[msg.payload["ufh_idx"]] = msg
+        elif msg.code == _0008:  # relay_demand, TODO: use msg DB?
+            if msg.payload.get("domain_id") == "FC":
+                self._relay_demand = msg
+            else:  # FA
+                self._relay_demand_fa = msg
 
-            # if dev_ids := msg.payload["devices"]:
-            #     if ctl := self._set_ctl(self._gwy._get_device(dev_ids[0])):
-            #         if zone := ctl._evo.zone_by_idx.get(msg.payload["zone_id"]):
-            #             zone.devices.append(self)
-            #             zone.device_by_id[self.id] = self
-            #             _LOGGER.debug("Device %s: added to Zone: %s", self, zone)
+        elif msg.code == _000C:  # zone_devices
+            if msg.payload["_device_class"] not in ("00", "04", "09"):
+                return  # ALL, SENsor, UFH
 
-        elif msg.code == _22C9:
+            ufh_idx = msg.payload["ufh_idx"]
+
+            if not msg.payload["zone_id"]:
+                self._circuits.pop(ufh_idx, None)
+                return
+            self._circuits[ufh_idx] = {"zone_idx": msg.payload["zone_id"]}
+
+            if dev_ids := msg.payload["devices"]:
+                # self._circuits[ufh_idx]["devices"] = dev_ids[0]  # or:
+                if ctl := self._set_ctl(self._gwy._get_device(dev_ids[0])):
+                    # self._circuits[ufh_idx]["devices"] = ctl.id  # better
+                    self._set_parent(ctl._evo._get_zone(msg.payload["zone_id"]), msg)
+
+        elif msg.code == _22C9:  # ufh_setpoints
             #  I --- 02:017205 --:------ 02:017205 22C9 024 00076C0A280101076C0A28010...
             #  I --- 02:017205 --:------ 02:017205 22C9 006 04076C0A2801
             self._setpoints = msg
 
-        elif msg.code == _3150:
-            if isinstance(msg.payload, list):
+            if self._gwy.config.enable_eavesdrop:  # and...:
+                self._eavesdrop_ufc_circuits(msg)
+
+        elif msg.code == _3150:  # heat_demands
+            if isinstance(msg.payload, list):  # the circuit demands
                 self._heat_demands = msg
             elif msg.payload.get("domain_id") == "FC":
                 self._heat_demand = msg
@@ -1157,42 +1190,61 @@ class UfhController(Device):  # UFC (02):
             ):
                 zone._handle_msg(msg)
 
+        # elif msg.code not in (_10E0, _22D0):
+        #     print("AAA")
+
         # "0008|FA/FC", "22C9|array", "22D0|none", "3150|ZZ/array(/FC?)"
 
+    def _eavesdrop_ufc_circuits(self, msg):
+        # assert msg.code == _22C9
+        circuits = [c["ufh_idx"] for c in msg.payload]
+
+        for ufh_idx in [f"{x:02X}" for x in range(8)]:
+            if ufh_idx not in self._circuits and ufh_idx in circuits:
+                self._circuits[ufh_idx] = {"zone_idx": None}
+            elif ufh_idx in self._circuits and ufh_idx not in circuits:
+                pass  # TODO: ?.pop()
+
+    def _set_parent(self, parent, domain=None) -> None:
+        self._set_ctl(parent._ctl)
+
+        if self not in parent.devices:
+            parent.devices.append(self)
+            parent.device_by_id[self.id] = self
+
+        return parent
+
     @property
     def circuits(self) -> Optional[Dict]:  # 000C
-        return {
-            k: {"zone_idx": m.payload["zone_id"]} for k, m in self._circuits.items()
-        }
+        return self._circuits
 
     @property
-    def zones(self) -> Optional[Dict]:  # 000C
-        return {m.payload["zone_id"]: {"ufh_idx": k} for k, m in self._circuits.items()}
-
-        # def fix(k):
-        #     return "zone_idx" if k == "zone_id" else k
-
-        # return [
-        #     {fix(k): v for k, v in m.payload.items() if k in ("ufh_idx", "zone_id")}
-        #     for m in self._circuits.values()
-        # ]
-
-    @property
-    def heat_demand(self) -> Optional[float]:  # 3150
+    def heat_demand(self) -> Optional[float]:  # 3150|FC (there is also 3150|FA)
         if self._heat_demand:
             return self._heat_demand.payload[self.HEAT_DEMAND]
 
     @property
-    def relay_demand(self) -> Optional[Dict]:  # 0008
-        return self._msg_value(_0008, key=ATTR_RELAY_DEMAND)
+    def heat_demands(self) -> Optional[dict]:  # 3150|ufh_idx array
+        if self._heat_demands:
+            return self._heat_demands.payload
 
     @property
-    def setpoints(self) -> Optional[Dict]:  # 22C9
+    def relay_demand(self) -> Optional[Dict]:  # 0008|FC
+        if self._relay_demand:
+            return self._relay_demand.payload[ATTR_RELAY_DEMAND]
+
+    @property
+    def relay_demand_fa(self) -> Optional[Dict]:  # 0008|FA
+        if self._relay_demand_fa:
+            return self._relay_demand_fa.payload[ATTR_RELAY_DEMAND]
+
+    @property
+    def setpoints(self) -> Optional[Dict]:  # 22C9|ufh_idx array
         if self._setpoints is None:
             return
 
         return {
-            c["ufh_idx"]: {"temp_high": c["temp_high"], "temp_low": c["temp_low"]}
+            c["ufh_idx"]: {k: v for k, v in c.items() if k in ("temp_low", "temp_high")}
             for c in self._setpoints.payload
         }
 
@@ -1216,6 +1268,7 @@ class UfhController(Device):  # UFC (02):
             **super().status,
             ATTR_HEAT_DEMAND: self.heat_demand,
             ATTR_RELAY_DEMAND: self.relay_demand,
+            "relay_demand_fa": self.relay_demand_fa,
         }
 
 
@@ -1255,10 +1308,10 @@ class DhwSensor(DhwTemperature, BatteryState, Device):  # DHW (07): 10A0, 1260
         }
 
 
-class ExtSensor(Weather, Device):  # EXT: 17
-    """The EXT class (external sensor), such as a HB85/HB95."""
+class OutSensor(Weather, Device):  # OUT: 17
+    """The OUT class (external sensor), such as a HB85/HB95."""
 
-    _DEV_KLASS = DEV_KLASS.EXT
+    _DEV_KLASS = DEV_KLASS.OUT
     _DEV_TYPES = ("17",)
 
     # LUMINOSITY = "luminosity"  # lux
@@ -1278,6 +1331,20 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
 
     _STATE_ATTR = "rel_modulation_level"
 
+    _CODE_MAP = {
+        # "00": _3EF0,  # master/slave status (actuator_state)
+        "01": _22D9,  # boiler_setpoint
+        "11": _3EF0,  # rel_modulation_level (actuator_state, also _3EF1)
+        "12": _1300,  # ch_water_pressure
+        "13": _12F0,  # dhw_flow_rate
+        "19": _3200,  # boiler_output_temp
+        "1A": _1260,  # dhw_temp
+        "1B": _1290,  # outside_temp
+        "1C": _3210,  # boiler_return_temp
+        "38": _10A0,  # dhw_setpoint (is a PARAM)
+        "39": _1081,  # ch_max_setpoint (is a PARAM)
+    }
+
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
@@ -1285,7 +1352,8 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
 
         self._msgz[_3220] = {RP: {}}
         self._opentherm_msg = self._msgz[_3220][RP]
-        self._supported_msg = {}
+        self._supported_msgs = {}
+        self._supported_codes = {}
         # self._ctl_polled_msg = {}
 
     def _start_discovery(self) -> None:
@@ -1312,28 +1380,41 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m))
                 for m in SCHEMA_MSG_IDS  # From OT v2.2: version numbers
-                if self._supported_msg.get(m) is not False
+                if self._supported_msgs.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
 
         if discover_flag & Discover.PARAMS:
-            [
-                self._send_cmd(Command.get_opentherm_data(self.id, m))
-                for m in PARAMS_MSG_IDS
-                if self._supported_msg.get(m) is not False
-                and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
-            ]
+            if OTB_MODE:
+                [
+                    self._send_cmd(Command.get_opentherm_data(self.id, m))
+                    for m in PARAMS_MSG_IDS
+                    if self._supported_msgs.get(m) is not False
+                ]
+                return
+
+            for code in [v for k, v in self._CODE_MAP.items() if k in PARAMS_MSG_IDS]:
+                if self._supported_codes.get(code) is not False:
+                    self._send_cmd(Command(RQ, code, "00", self.id, retries=0))
 
         if discover_flag & Discover.STATUS:
-            [
-                self._send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
-                for m in STATUS_MSG_IDS
-                if self._supported_msg.get(m) is not False
-                # nd (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
-            ]  # TODO: add expired
+            self._send_cmd(Command(RQ, _2401, "00", self.id))  # WIP
+            self._send_cmd(Command(RQ, _3EF0, "00", self.id))
 
-            self._send_cmd(Command(RQ, _2401, "00", self.id))
-            self._send_cmd(Command(RQ, _3EF0, "00", self.id))  # CTLs dont RP to RQ/3EF0
+            if OTB_MODE:
+                for msg_id in STATUS_MSG_IDS:
+                    if self._supported_msgs.get(msg_id) is not False:
+                        self._send_cmd(
+                            Command.get_opentherm_data(self.id, msg_id, retries=0)
+                        )
+                return
+
+            self._send_cmd(Command.get_opentherm_data(self.id, "00"))
+            self._send_cmd(Command.get_opentherm_data(self.id, "73"))
+
+            for code in [v for k, v in self._CODE_MAP.items() if k in STATUS_MSG_IDS]:
+                if self._supported_codes.get(code) is not False:
+                    self._send_cmd(Command(RQ, code, "00", self.id, retries=0))
 
         if False and DEV_MODE and discover_flag & Discover.STATUS:
             # TODO: these are WIP, and do vary in payload
@@ -1358,59 +1439,65 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
-        if msg.code != _3220:
-            return
+        if msg.code == _3220:
+            self._handle_3220(msg)
+        if msg.code in self._CODE_MAP.values():
+            self._handle_code(msg)
 
+    def _handle_3220(self, msg) -> None:
         msg_id = f"{msg.payload[MSG_ID]:02X}"
 
-        if DEV_MODE and msg_id != "73":
+        if DEV_MODE and msg_id != "73":  # oem code, here to follow state changes
             self._send_cmd(Command.get_opentherm_data(self.id, "73"))
 
-        # if msg.dst is self._ctl:
-        #     if msg_id not in self._ctl_polled_msg:
-        #         self._ctl_polled_msg[msg_id] = None
+        # TODO: this is development code - will be rationalised, eventually
+        if OTB_MODE and (code := self._CODE_MAP.get(msg_id)):
+            self._send_cmd(Command(RQ, code, "00", self.id, retries=0))
 
-        #     elif self._ctl_polled_msg[msg_id] is None:
-        #         self._ctl_polled_msg[msg_id] = True
-        #         _LOGGER.warning(
-        #             f"{msg._pkt} < OpenTherm: deprecating msg_id "
-        #             f"0x{msg_id}: it appears polled by the controller",
-        #         )
+        if msg._pkt.payload[6:] == "47AB" or msg._pkt.payload[4:] == "121980":
+            if msg_id not in self._supported_msgs:
+                self._supported_msgs[msg_id] = None
 
-        if msg._pkt.payload[4:] == "121980" or msg._pkt.payload[6:] == "47AB":
-            if msg_id not in self._supported_msg:
-                self._supported_msg[msg_id] = None
-
-            elif self._supported_msg[msg_id] is None:
-                self._supported_msg[msg_id] = False
+            elif self._supported_msgs[msg_id] is None:
+                self._supported_msgs[msg_id] = False
                 _LOGGER.warning(
-                    f"{msg._pkt} < OpenTherm: deprecating msg_id "
+                    f"{msg._pkt} < OTB: deprecating msg_id "
                     f"0x{msg_id}: it appears unsupported",
                 )
 
         else:
-            self._supported_msg[msg_id] = msg.payload[MSG_TYPE] not in (
+            self._supported_msgs[msg_id] = msg.payload[MSG_TYPE] not in (
                 "Data-Invalid",
                 "Unknown-DataId",
                 "-reserved-",
             )
 
-        # TODO: this is development code - will be rationalised, eventually
-        code = {
-            # "00": _3EF0,  # master/slave status
-            "01": _22D9,  # boiler_setpoint
-            "11": _3EF1,  # rel_modulation_level (also _3EF0)
-            "12": _1300,  # ch_water_pressure
-            "13": _12F0,  # dhw_flow_rate
-            "19": _3200,  # boiler_output_temp - checked
-            "1A": _1260,  # dhw_temp - checked
-            "1B": _1290,  # outside_temp
-            "1C": _3210,  # boiler_return_temp - checked
-            "38": _10A0,  # dhw_setpoint (is a PARAM)
-            "39": _1081,  # ch_max_setpoint (is a PARAM)
-        }.get(msg_id)
-        if code:
-            self._send_cmd(Command(RQ, code, "00", self.id, retries=0))
+    def _handle_code(self, msg) -> None:
+        if DEV_MODE and msg.code != _2401:  # unknown, here to follow state changes
+            self._send_cmd(Command(RQ, _2401, "00", self.id))
+
+        if msg.code in (_10A0, _3EF0, _3EF1) or msg.len != 3:
+            return
+
+        if msg._pkt.payload[2:] == "7FFF" or (
+            msg.code == _1300 and msg._pkt.payload[2:] == "09F6"
+        ):
+            if msg.code not in self._supported_codes:
+                self._supported_codes[msg.code] = None
+
+            elif self._supported_codes[msg.code] is None:
+                self._supported_codes[msg.code] = False
+                _LOGGER.warning(
+                    f"{msg._pkt} < OTB: deprecating code "
+                    f"0x{msg.code}: it appears unsupported",
+                )
+
+        else:
+            self._supported_codes.pop(msg.code, None)
+
+    def _ot_msg_flag(self, msg_id, flag_idx) -> Optional[bool]:
+        if flags := self._ot_msg_value(msg_id):
+            return bool(flags[flag_idx])
 
     @staticmethod
     def _ot_msg_name(msg) -> str:
@@ -1423,183 +1510,142 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     def _ot_msg_value(self, msg_id) -> Optional[float]:
         if (
             (msg := self._opentherm_msg.get(msg_id))
-            and self._supported_msg[msg_id]
+            and self._supported_msgs[msg_id]
             and not msg._expired
         ):
             return msg.payload.get(VALUE)
 
     @property
-    def _bit_2_4(self) -> Optional[bool]:  # 2401
-        if flags := self._msg_value(_2401, key="_flags_2"):
-            return flags[4]
-
-    # @property
-    # def num_starts_burner(self) -> Optional[int]:  # 3220/74
-    #     self._ot_msg_value("74")
+    def bit_2_4(self) -> Optional[bool]:  # 2401 - WIP
+        return self._msg_flag(_2401, "_flags_2", 4)
 
     @property
-    def _bit_2_5(self) -> Optional[bool]:  # 2401
-        if flags := self._msg_value(_2401, key="_flags_2"):
-            return flags[5]
+    def bit_2_5(self) -> Optional[bool]:  # 2401 - WIP
+        return self._msg_flag(_2401, "_flags_2", 5)
 
     @property
-    def _bit_2_6(self) -> Optional[bool]:  # 2401
-        if flags := self._msg_value(_2401, key="_flags_2"):
-            return flags[6]
+    def bit_2_6(self) -> Optional[bool]:  # 2401 - WIP
+        return self._msg_flag(_2401, "_flags_2", 6)
 
     @property
-    def _bit_2_7(self) -> Optional[bool]:  # 2401
-        if flags := self._msg_value(_2401, key="_flags_2"):
-            return flags[7]
+    def bit_2_7(self) -> Optional[bool]:  # 2401 - WIP
+        return self._msg_flag(_2401, "_flags_2", 7)
 
     @property
-    def _oem_code(self) -> Optional[float]:  # 3220/73
+    def oem_code(self) -> Optional[float]:  # 3220/73
         return self._ot_msg_value("73")
 
     @property
-    def _percent(self) -> Optional[float]:  # 2401 - WIP
+    def percent(self) -> Optional[float]:  # 2401 - WIP
         return self._msg_value(_2401, key="_percent_3")
 
     @property
-    def _value(self) -> Optional[int]:  # 2401 - WIP
+    def value(self) -> Optional[int]:  # 2401 - WIP
         return self._msg_value(_2401, key="_value_2")
 
     @property
-    def boiler_output_temp(self) -> Optional[float]:  # 3220/19 (3200)
-        return self._ot_msg_value("19")
-
-    @property
-    def _boiler_output_temp(self) -> Optional[float]:  # 3200 (3220/19)
+    def boiler_output_temp(self) -> Optional[float]:  # 3200 (3220/19)
+        if OTB_MODE:
+            return self._ot_msg_value("19")
         return self._msg_value(_3200, key="temperature")
 
     @property
-    def boiler_return_temp(self) -> Optional[float]:  # 3220/1C (3210)
-        return self._ot_msg_value("1C")
-
-    @property
-    def _boiler_return_temp(self) -> Optional[float]:  # 3210 (3220/1C)
+    def boiler_return_temp(self) -> Optional[float]:  # 3210 (3220/1C)
+        if OTB_MODE:
+            return self._ot_msg_value("1C")
         return self._msg_value(_3210, key="temperature")
 
     @property
-    def boiler_setpoint(self) -> Optional[float]:  # 3220/01 (22D9)
-        return self._ot_msg_value("01")
-
-    @property
-    def _boiler_setpoint(self) -> Optional[float]:  # 22D9
+    def boiler_setpoint(self) -> Optional[float]:  # 22D9 (3220/01)
+        if OTB_MODE:
+            return self._ot_msg_value("01")
         return self._msg_value(_22D9, key="setpoint")
 
     @property
-    def ch_max_setpoint(self) -> Optional[float]:  # 3220/39 (1081)
-        return self._ot_msg_value("39")
-
-    @property
-    def _ch_max_setpoint(self) -> Optional[float]:  # 1081
+    def ch_max_setpoint(self) -> Optional[float]:  # 1081 (3220/39)
+        if OTB_MODE:
+            return self._ot_msg_value("39")
         return self._msg_value(_1081, key="setpoint")
 
     @property
-    def _ch_setpoint(self) -> Optional[bool]:  # 3EF0
-        return self._msg_value(_3EF0, key="ch_setpoint")
-
-    @property
-    def ch_water_pressure(self) -> Optional[float]:  # 3220/12 (1300)
-        return self._ot_msg_value("12")
-
-    @property
-    def _ch_water_pressure(self) -> Optional[float]:  # 1300
+    def ch_water_pressure(self) -> Optional[float]:  # 1300 (3220/12)
+        if OTB_MODE:
+            return self._ot_msg_value("12")
         result = self._msg_value(_1300, key="pressure")
         return None if result == 25.5 else result  # HACK: to make more rigourous
 
     @property
-    def dhw_flow_rate(self) -> Optional[float]:  # 3220/13 (12F0)
-        return self._ot_msg_value("13")
-
-    @property
-    def _dhw_flow_rate(self) -> Optional[float]:  # 12F0
+    def dhw_flow_rate(self) -> Optional[float]:  # 12F0 (3220/13)
+        if OTB_MODE:
+            return self._ot_msg_value("13")
         return self._msg_value(_12F0, key="dhw_flow_rate")
 
     @property
-    def dhw_setpoint(self) -> Optional[float]:  # 10A0
-        return self._ot_msg_value("38")
+    def dhw_setpoint(self) -> Optional[float]:  # 10A0 (3220/38)
+        if OTB_MODE:
+            return self._ot_msg_value("38")
+        return self._msg_value(_10A0, key="setpoint")
 
     @property
-    def _dhw_setpoint(self) -> Optional[float]:  # 3220/38 (10A0)
-        return self._msg_value(_1300, key="setpoint")
-
-    @property
-    def dhw_temp(self) -> Optional[float]:  # 3220/1A (1260)
-        return self._ot_msg_value("1A")
-
-    @property
-    def _dhw_temp(self) -> Optional[float]:  # 1260
+    def dhw_temp(self) -> Optional[float]:  # 1260 (3220/1A)
+        if OTB_MODE:
+            return self._ot_msg_value("1A")
         return self._msg_value(_1260, key="temperature")
 
     @property
-    def outside_temp(self) -> Optional[float]:  # 3220/1B (1290)
-        return self._ot_msg_value("1B")
-
-    @property
-    def _outside_temp(self) -> Optional[float]:  # 1290
+    def outside_temp(self) -> Optional[float]:  # 1290 (3220/1B)
+        if OTB_MODE:
+            return self._ot_msg_value("1B")
         return self._msg_value(_1290, key="temperature")
 
     @property
-    def rel_modulation_level(self) -> Optional[float]:  # 3220/11 (3EFx)
-        """Return the relative modulation level from OpenTherm."""
-        return self._ot_msg_value("11")
-
-    @property
-    def _rel_modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
+    def rel_modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
         """Return the relative modulation level from RAMSES_II."""
         return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
 
     @property
-    def _max_rel_modulation(self) -> Optional[float]:  # 3EF0
-        return self._msg_value(_3EF0, key="max_rel_modulation")
+    def rel_modulation_level_ot(self) -> Optional[float]:  # 3220/11
+        """Return the relative modulation level from OpenTherm."""
+        return self._ot_msg_value("11")
 
     @property
     def ch_active(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[8 + 1]
+        return self._ot_msg_flag("00", 8 + 1) if OTB_MODE else super().ch_active
 
     @property
     def ch_enabled(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[0]
+        return self._ot_msg_flag("00", 0) if OTB_MODE else super().ch_enabled
 
     @property
     def dhw_active(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[8 + 2]
+        return self._ot_msg_flag("00", 8 + 2) if OTB_MODE else super().dhw_active
 
     @property
     def dhw_enabled(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[1]
+        return self._ot_msg_flag("00", 1)  # if OTB_MODE else None  # TODO: super().xxx
 
     @property
     def flame_active(self) -> Optional[bool]:  # 3220/00 (flame_on)
-        if flags := self._ot_msg_value("00"):
-            return flags[8 + 3]
+        return self._ot_msg_flag("00", 8 + 3) if OTB_MODE else super().flame_active
 
     @property
     def cooling_active(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[8 + 4]
+        return self._ot_msg_flag("00", 8 + 4)  # if OTB_MODE else None  # TODO: super...
 
     @property
     def cooling_enabled(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[2]
+        return self._ot_msg_flag("00", 2)  # if OTB_MODE else None  # TODO: super().xxx
 
     @property
     def fault_present(self) -> Optional[bool]:  # 3220/00
-        if flags := self._ot_msg_value("00"):
-            return flags[8]
+        return self._ot_msg_flag("00", 8)  # if OTB_MODE else None  # TODO: super().xxx
 
+    @property
     def opentherm_schema(self) -> dict:
         result = {
             self._ot_msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
-            if self._supported_msg.get(int(k, 16)) and int(k, 16) in SCHEMA_MSG_IDS
+            if self._supported_msgs.get(int(k, 16)) and int(k, 16) in SCHEMA_MSG_IDS
         }
         return {
             m: {k: v for k, v in p.items() if k.startswith(VALUE)}
@@ -1608,17 +1654,18 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
 
     @property
     def opentherm_counters(self) -> dict:
+        # for msg_id in ("71", "72", ...):
         return {
-            "starts_failed": self._ot_msg_value("71"),
-            "flame_signal_low": self._ot_msg_value("72"),
-            "starts_burner": self._ot_msg_value("74"),
-            "starts_ch_pump": self._ot_msg_value("75"),
-            "dhw_actuator": self._ot_msg_value("76"),
-            "starts_dhw": self._ot_msg_value("77"),
-            "operating_hours": self._ot_msg_value("78"),
+            "burner_hours": self._ot_msg_value("78"),
+            "burner_starts": self._ot_msg_value("74"),
+            "burner_failed_starts": self._ot_msg_value("71"),
             "ch_pump_hours": self._ot_msg_value("79"),
+            "ch_pump_starts": self._ot_msg_value("75"),
+            "dhw_burner_hours": self._ot_msg_value("7B"),
+            "dhw_burner_starts": self._ot_msg_value("77"),
             "dhw_pump_hours": self._ot_msg_value("7A"),
-            "hdw_burner_hours": self._ot_msg_value("7B"),
+            "dhw_pump_starts": self._ot_msg_value("76"),
+            "flame_signal_low": self._ot_msg_value("72"),
         }
 
     @property
@@ -1626,7 +1673,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         result = {
             self._ot_msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
-            if self._supported_msg.get(int(k, 16)) and int(k, 16) in PARAMS_MSG_IDS
+            if self._supported_msgs.get(int(k, 16)) and int(k, 16) in PARAMS_MSG_IDS
         }
         return {
             m: {k: v for k, v in p.items() if k.startswith(VALUE)}
@@ -1636,39 +1683,54 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     @property
     def opentherm_status(self) -> dict:
         return {
-            "boiler_output_temp": self.boiler_output_temp,
-            "boiler_return_temp": self.boiler_return_temp,
-            "boiler_setpoint": self.boiler_setpoint,
-            "ch_max_setpoint": self.ch_max_setpoint,
-            "ch_water_pressure": self.ch_water_pressure,
-            "dhw_flow_rate": self.dhw_flow_rate,
-            "dhw_setpoint": self.dhw_setpoint,
-            "dhw_temp": self.dhw_temp,
-            "outside_temp": self.outside_temp,
-            "rel_modulation_level": self.rel_modulation_level,
+            "boiler_output_temp": self._ot_msg_value("19"),
+            "boiler_return_temp": self._ot_msg_value("1C"),
+            "boiler_setpoint": self._ot_msg_value("01"),
+            "ch_max_setpoint": self._ot_msg_value("39"),
+            "ch_water_pressure": self._ot_msg_value("12"),
+            "dhw_flow_rate": self._ot_msg_value("13"),
+            "dhw_setpoint": self._ot_msg_value("38"),
+            "dhw_temp": self._ot_msg_value("1A"),
+            "outside_temp": self._ot_msg_value("1B"),
+            "rel_modulation_level": self.rel_modulation_level_ot,
+            #
+            "oem_code": self._ot_msg_value("73"),
+            #
+            "ch_active": self._ot_msg_flag("00", 8 + 1),
+            "ch_enabled": self._ot_msg_flag("00", 0),
+            "cooling_active": self._ot_msg_flag("00", 8 + 4),
+            "cooling_enabled": self._ot_msg_flag("00", 2),
+            "dhw_active": self._ot_msg_flag("00", 8 + 2),
+            "dhw_enabled": self._ot_msg_flag("00", 1),
+            "fault_present": self._ot_msg_flag("00", 8),
+            "flame_active": self._ot_msg_flag("00", 8 + 3),
         }
 
     @property
     def ramses_params(self) -> dict:
         return {
-            "ch_max_setpoint": self._ch_max_setpoint,
-            "dhw_setpoint": self._dhw_setpoint,
-            "max_rel_modulation": self._max_rel_modulation,
+            "max_rel_modulation": self.max_rel_modulation,
         }
 
     @property
     def ramses_status(self) -> dict:
         return {
-            "boiler_setpoint": self._boiler_setpoint,
-            "ch_setpoint": self._ch_setpoint,
-            "ch_water_pressure": self._ch_water_pressure,
-            "dhw_temp": self._dhw_temp,
-            "outside_temp": self._outside_temp,
-            "rel_modulation_level": self._rel_modulation_level,
-            "ch_active": self._ch_active,
-            "ch_enabled": self._ch_enabled,
-            "dhw_active": self._dhw_active,
-            "flame_active": self._flame_active,
+            "boiler_output_temp": self._msg_value(_3200, key="temperature"),
+            "boiler_return_temp": self._msg_value(_3210, key="temperature"),
+            "boiler_setpoint": self._msg_value(_22D9, key="setpoint"),
+            "ch_max_setpoint": self._msg_value(_1081, key="setpoint"),
+            "ch_water_pressure": self._msg_value(_1300, key="pressure"),
+            "dhw_flow_rate": self._msg_value(_12F0, key="dhw_flow_rate"),
+            "dhw_setpoint": self._msg_value(_1300, key="setpoint"),
+            "dhw_temp": self._msg_value(_1260, key="temperature"),
+            "outside_temp": self._msg_value(_1290, key="temperature"),
+            "rel_modulation_level": self.rel_modulation_level,
+            #
+            "ch_setpoint": super().ch_setpoint,
+            "ch_active": super().ch_active,
+            "ch_enabled": super().ch_enabled,
+            "dhw_active": super().dhw_active,
+            "flame_active": super().flame_active,
         }
 
     @property
@@ -1677,7 +1739,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
             **super().schema,
             "known_msg_ids": {
                 k: OPENTHERM_MESSAGES[int(k, 16)].get("var", k)
-                for k, v in sorted(self._supported_msg.items())
+                for k, v in sorted(self._supported_msgs.items())
                 if v
             },
             "opentherm_schema": self.opentherm_schema,
@@ -1688,7 +1750,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         return {
             **super().params,
             "opentherm_params": self.opentherm_params,
-            "supported_msgs": dict(sorted(self._supported_msg.items())),
+            "supported_msgs": dict(sorted(self._supported_msgs.items())),
         }
 
     @property
@@ -2077,7 +2139,7 @@ _DEV_TYPE_TO_KLASS = {  # TODO: *remove*
     "10": DEV_KLASS.OTB,
     "12": DEV_KLASS.THM,  # 12: can act like a DEV_KLASS.PRG
     "13": DEV_KLASS.BDR,
-    "17": DEV_KLASS.EXT,
+    "17": DEV_KLASS.OUT,
     "18": DEV_KLASS.HGI,
     "22": DEV_KLASS.THM,  # 22: can act like a DEV_KLASS.PRG
     "23": DEV_KLASS.PRG,
