@@ -10,13 +10,21 @@ import asyncio
 import functools
 import json
 import logging
+import re
 from datetime import datetime as dt
 from datetime import timedelta as td
 from types import SimpleNamespace
 from typing import Any, Optional, Tuple, Union
 
 from .address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR, Address, pkt_addrs
-from .const import COMMAND_REGEX, SYSTEM_MODE, ZONE_MODE
+from .const import (
+    COMMAND_REGEX,
+    DEVICE_ID_REGEX,
+    HGI_DEVICE_ID,
+    NON_DEVICE_ID,
+    SYSTEM_MODE,
+    ZONE_MODE,
+)
 from .exceptions import ExpiredCallbackError, InvalidPacketError
 from .frame import PacketBase, pkt_header
 from .helpers import dt_now, dtm_to_hex, str_to_hex, temp_to_hex, timestamp
@@ -38,6 +46,7 @@ from .const import (  # noqa: F401, isort: skip
     _000E,
     _0016,
     _0100,
+    _0150,
     _01D0,
     _01E9,
     _0404,
@@ -46,9 +55,13 @@ from .const import (  # noqa: F401, isort: skip
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
+    _1098,
     _10A0,
+    _10B0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -58,9 +71,12 @@ from .const import (  # noqa: F401, isort: skip
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
+    _1FD0,
     _1FD4,
     _2249,
     _22C9,
@@ -70,6 +86,11 @@ from .const import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
+    _2400,
+    _2401,
+    _2410,
+    _2420,
     _2D49,
     _2E04,
     _30C9,
@@ -79,7 +100,11 @@ from .const import (  # noqa: F401, isort: skip
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
+    _3221,
+    _3223,
     _3B00,
     _3EF0,
     _3EF1,
@@ -88,6 +113,7 @@ from .const import (  # noqa: F401, isort: skip
 
 COMMAND_FORMAT = "{:<2} {} {} {} {} {} {:03d} {}"
 
+DEVICE_ID_REGEX = re.compile(DEVICE_ID_REGEX.ANY)
 
 TIMER_SHORT_SLEEP = 0.05
 TIMER_LONG_TIMEOUT = td(seconds=60)
@@ -122,7 +148,8 @@ QOS_TABLE = {  # priority, retries, timeout, disable_backoff, c.f. QOS_TX_DEFAUL
     f"{I_}/{_0404}": (Priority.HIGH, 5, td(seconds=0.30), None),  # TODO: both short Tx,
     f"{W_}/{_0404}": (Priority.HIGH, 5, td(seconds=0.30), None),  # TODO: but long Rx
     f"{RQ}/{_0418}": (Priority.LOW, 3, None, None),
-    f"{RQ}/{_3220}": (Priority.DEFAULT, 1, td(seconds=1), True),
+    f"{RQ}/{_3220}": (Priority.DEFAULT, 1, td(seconds=1.2), True),
+    f"{W_}/{_3220}": (Priority.HIGH, 3, td(seconds=1.2), True),
 }  # The long timeeout for the OTB is for total RTT to slave (boiler)
 
 
@@ -826,7 +853,7 @@ class Command(PacketBase):
 
         return cmd
 
-    @classmethod  # constructor for RP/3EF1 (TODO: & I/3EF1?)
+    @classmethod  # constructor for RP/3EF1 (I/3EF1?)  # TODO: trap corrupt values?
     @validate_api_params()
     def put_actuator_cycle(
         cls,
@@ -843,14 +870,17 @@ class Command(PacketBase):
         """
         # RP --- 13:049798 18:006402 --:------ 3EF1 007 00-0126-0126-00-FF
 
-        assert src_id[:2] == "13"  # dex
+        try:  # dex
+            assert src_id[:2] == "13", "device_id should be like 13:xxxxxx"
+        except AssertionError as exc:
+            raise TypeError(f"Faked device {src_id} has unsupported device type: {exc}")
 
         payload = f"00{actuator_countdown:04X}"
         payload += f"{cycle_countdown:04X}" if cycle_countdown is not None else "7FFF"
         payload += f"{int(mod_level * 200):02X}FF"
         return cls.packet(RP, _3EF1, payload, addr0=src_id, addr1=dst_id, **kwargs)
 
-    @classmethod  # constructor for I/3EF0
+    @classmethod  # constructor for I/3EF0  # TODO: trap corrupt states?
     @validate_api_params()
     def put_actuator_state(cls, dev_id: str, mod_level: float, **kwargs):
         """Constructor to announce the modulation level of an actuator (3EF0).
@@ -860,7 +890,10 @@ class Command(PacketBase):
         #  I --- 13:049798 --:------ 13:049798 3EF0 003 00C8FF
         #  I --- 13:106039 --:------ 13:106039 3EF0 003 0000FF
 
-        assert dev_id[:2] == "13"  # dex
+        try:  # dex
+            assert dev_id[:2] == "13", "device_id should be like 13:xxxxxx"
+        except AssertionError as exc:
+            raise TypeError(f"Faked device {dev_id} has unsupported device type: {exc}")
 
         payload = "007FFF" if mod_level is None else f"00{int(mod_level * 200):02X}FF"
         return cls.packet(I_, _3EF0, payload, addr0=dev_id, addr2=dev_id, **kwargs)
@@ -892,7 +925,23 @@ class Command(PacketBase):
             verb, _1FC9, payload, addr0=src_id, addr1=dst_id, addr2=addr2, **kwargs
         )
 
-    @classmethod  # constructor for I/0002
+    @classmethod  # constructor for I/1260  # TODO: trap corrupt temps?
+    @validate_api_params()
+    def put_dhw_temp(cls, dev_id: str, temperature: float, **kwargs):
+        """Constructor to announce the current temperature of an DHW sensor (1260).
+
+        This is for use by a faked CS92A or similar.
+        """
+
+        try:  # dex
+            assert dev_id[:2] == "07", "device_id should be like 07:xxxxxx"
+        except AssertionError as exc:
+            raise TypeError(f"Faked device {dev_id} has unsupported device type: {exc}")
+
+        payload = f"00{temp_to_hex(temperature)}"
+        return cls.packet(I_, _1260, payload, addr0=dev_id, addr2=dev_id, **kwargs)
+
+    @classmethod  # constructor for I/0002  # TODO: trap corrupt temps?
     @validate_api_params()
     def put_outdoor_temp(cls, dev_id: str, temperature: float, **kwargs):
         """Constructor to announce the current temperature of an outdoor sensor (0002).
@@ -900,22 +949,27 @@ class Command(PacketBase):
         This is for use by a faked HB85 or similar.
         """
 
-        assert dev_id[:2] == "17"  # dex
+        try:  # dex
+            assert dev_id[:2] == "17", "device_id should be like 17:xxxxxx"
+        except AssertionError as exc:
+            raise TypeError(f"Faked device {dev_id} has unsupported device type: {exc}")
 
         payload = f"00{temp_to_hex(temperature)}01"
         return cls.packet(I_, _0002, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
-    @classmethod  # constructor for I/30C9
+    @classmethod  # constructor for I/30C9  # TODO: trap corrupt temps?
     @validate_api_params()
     def put_sensor_temp(cls, dev_id: str, temperature: float, **kwargs):
         """Constructor to announce the current temperature of a thermostat (3C09).
 
-        This is for use by a faked BDR91A or similar.
+        This is for use by a faked DTS92(E) or similar.
         """
         #  I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
 
-        if dev_id[:2] not in ("03", "12", "22", "34"):  # dex
-            raise TypeError(f"Wrong device type: {dev_id[:2]}, try 03, 12, 22, 34")
+        try:  # dex - use 03 to avoid device_id collisions
+            assert dev_id[:2] == "03", "device_id should be like 03:xxxxxx"
+        except AssertionError as exc:
+            raise TypeError(f"Faked device {dev_id} has unsupported device type: {exc}")
 
         payload = f"00{temp_to_hex(temperature)}"
         return cls.packet(I_, _30C9, payload, addr0=dev_id, addr2=dev_id, **kwargs)
@@ -969,6 +1023,39 @@ class Command(PacketBase):
 
         return cls(I_, _PUZZ, payload[:48], NUL_DEV_ADDR.id, **kwargs)
 
+    @classmethod  # constructor for internal use only
+    def from_str(cls, cmd_str: str, **kwargs):
+        """Create a command from a string.
+
+        Examples include (whitespace for readability):
+            'RQ     01:123456               1F09 00'
+            'RQ     01:123456     13:123456 3EF0 00'
+            'RQ     07:045960     01:054173 10A0 00137400031C'
+            ' W 123 30:045960 -:- 32:054173 22F1 001374'
+        """
+
+        cmd = cmd_str.upper().split()
+        if len(cmd) < 4:
+            raise ValueError("Command is invalid: '%s'", cmd_str)
+
+        verb = cmd.pop(0)
+        seqn = "---" if DEVICE_ID_REGEX.match(cmd[0]) else cmd.pop(0)
+        payload = cmd.pop()[:48]
+        code = cmd.pop()
+
+        if not 0 < len(cmd) < 4:
+            raise ValueError("Command is invalid: '%s'", cmd_str)
+        elif len(cmd) == 1:
+            addrs = (HGI_DEVICE_ID, cmd[0], NON_DEVICE_ID)
+        elif len(cmd) == 3:
+            addrs = (cmd[0], cmd[1], cmd[2])
+        elif cmd[0] == cmd[1]:
+            addrs = (cmd[0], NON_DEVICE_ID, cmd[1])
+        else:
+            addrs = (cmd[0], cmd[1], NON_DEVICE_ID)
+
+        return cls.packet(verb, code, payload, *addrs, seqn=seqn, **kwargs)
+
 
 # A convenience dict
 CODE_API_MAP = {
@@ -989,6 +1076,7 @@ CODE_API_MAP = {
     f"{RQ}/{_1100}": Command.get_tpi_params,
     f"{W_}/{_1100}": Command.set_tpi_params,
     f"{RQ}/{_1260}": Command.get_dhw_temp,
+    f"{I_}/{_1260}": Command.put_dhw_temp,
     f"{RQ}/{_12B0}": Command.get_zone_window_state,
     f"{RQ}/{_1F41}": Command.get_dhw_mode,
     f"{W_}/{_1F41}": Command.set_dhw_mode,
