@@ -592,9 +592,18 @@ def parser_0418(payload, msg) -> Optional[dict]:
     if dts_from_hex(payload[18:30]) is None:  # a null log entry
         return {"log_entry": None}
 
-    assert payload[2:4] in _0418_FAULT_STATE, payload[2:4]  # C0 don't appear in UI?
-    assert payload[8:10] in _0418_FAULT_TYPE, payload[8:10]
-    assert payload[12:14] in _0418_DEVICE_CLASS, payload[12:14]
+    try:
+        assert payload[2:4] in _0418_FAULT_STATE, f"fault_state: {payload[2:4]}"
+        assert payload[8:10] in _0418_FAULT_TYPE, f"fault_type: {payload[8:10]}"
+        assert payload[12:14] in _0418_DEVICE_CLASS, f"device class: {payload[12:14]}"
+        # 1C: 'Comms fault, Actuator': seen with boiler relays
+        assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
+            payload[10:12] in ("1C", "F9", "FA", "FC")
+        ), f"domain_id: {payload[10:12]}"
+    except AssertionError as exc:
+        _LOGGER.warning(
+            f"{msg._pkt} < {_INFORM_DEV_MSG} ({exc}), with a photo of your fault log"
+        )
 
     result = {
         "timestamp": dts_from_hex(payload[18:30]),
@@ -606,17 +615,12 @@ def parser_0418(payload, msg) -> Optional[dict]:
     if payload[10:12] == "FC" and result["device_class"] == "actuator":
         result["device_class"] = ATTR_HTG_CONTROL  # aka Boiler relay
 
-    # 1C: 'Comms fault, Actuator': seen with boiler relays
-    assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
-        payload[10:12] in ("1C", "F9", "FA", "FC")
-    ), f"unexpected domain_id: {payload[10:12]}"
-
     if payload[12:14] != "00":  # TODO: Controller
         key_name = (
-            "zone_id"
+            "zone_id"  # NOTE: don't use zone_idx (for now)
             if int(payload[10:12], 16) < msg._gwy.config.max_zones
             else "domain_id"
-        )  # TODO: don't use zone_idx (for now)
+        )
         result.update({key_name: payload[10:12]})
 
     if payload[38:] == "000002":  # "00:000002 for Unknown?
@@ -648,10 +652,10 @@ def parser_042f(payload, msg) -> Optional[dict]:
     #  I --- 32:166025 --:------ 32:166025 042F 009 00-050E0B0C00111470
 
     return {
-        "counter_1": int(payload[2:6], 16),
-        "counter_2": int(payload[6:10], 16),
-        "counter_total": int(payload[10:14], 16),
-        "unknown_7": payload[14:],
+        "counter_1": f"0x{payload[2:6]}",
+        "counter_2": f"0x{payload[6:10]}",
+        "counter_3": f"0x{payload[10:14]}",
+        "unknown_7": f"0x{payload[14:]}",
     }
 
 
@@ -1466,7 +1470,7 @@ def parser_2420(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # hometronics _state (of unknwon)
+@parser_decorator  # hometronics _state (of unknown)
 def parser_2d49(payload, msg) -> dict:
 
     assert payload[2:] in ("0000", "C800"), _INFORM_DEV_MSG
@@ -1969,7 +1973,6 @@ def parser_3ef0(payload, msg) -> dict:
         }
 
     assert msg.len in (3, 6, 9)
-    assert payload[:2] == "00", f"byte 1: {payload[:2]}"
 
     if msg.len == 3:  # I|BDR|003
         # .I --- 13:042805 --:------ 13:042805 3EF0 003 0000FF
@@ -2030,9 +2033,9 @@ def parser_3ef0(payload, msg) -> dict:
         assert "_flags_6" not in result or (
             [result["_flags_6"][i] for i in (0, 1, 2, 3, 4, 5)] == [0] * 6
         ), result["_flags_6"]
-    except AssertionError:
+    except AssertionError as exc:
         _LOGGER.warning(
-            f"{msg._pkt} < {_INFORM_DEV_MSG}, with a description of your system"
+            f"{msg._pkt} < {_INFORM_DEV_MSG} ({exc}), with a description of your system"
         )
 
     return result
@@ -2065,12 +2068,12 @@ def parser_3ef1(payload, msg) -> dict:
         # ), f"byte 3: {payload[6:10]}"
         assert percent(payload[10:12]) in (0, 1), f"byte 5: {payload[10:12]}"
 
-    else:  # is OTB?
+    else:  # is OTB
         # assert (
         #     re.compile(r"^00[0-9A-F]{10}10").match(payload)
         # ), "doesn't match: " + r"^00[0-9A-F]{10}10"
         assert payload[2:6] == "7FFF", f"byte 1: {payload[2:6]}"
-        assert payload[6:10] == "003C", f"byte 3: {payload[6:10]}"  # 1 minute
+        assert payload[6:10] == "003C", f"byte 3: {payload[6:10]}"  # 60 seconds
         assert percent(payload[10:12]) <= 1, f"byte 5: {payload[10:12]}"
 
     cycle_countdown = None if payload[2:6] == "7FFF" else int(payload[2:6], 16)
