@@ -806,58 +806,6 @@ RAMSES_DEVICES_CLASS = {
         _3EF0: {RQ: {}},  # when bound direct to a 13:
         _3EF1: {RQ: {}},  # when bound direct to a 13:
     },
-    # DEV_KLASS.RND: {  # e.g. TR87RF: Single (round) Zone Thermostat
-    #     _0005: {I_: {}},
-    #     _0008: {I_: {}},
-    #     _000A: {I_: {}, RQ: {}},
-    #     _000C: {I_: {}},
-    #     _000E: {I_: {}},
-    #     _042F: {I_: {}},
-    #     _1060: {I_: {}},
-    #     _10E0: {I_: {}},
-    #     _12C0: {I_: {}},
-    #     _1FC9: {I_: {}},
-    #     _2309: {I_: {}, RQ: {}, W_: {}},
-    #     _2349: {RQ: {}},
-    #     _30C9: {I_: {}},
-    #     _3120: {I_: {}},
-    #     _313F: {I_: {}},  # W --- 30:253184 34:010943 --:------ 313F 009 006000070E0...
-    #     _3EF0: {RQ: {}},  # when bound direct to a 13:
-    #     _3EF1: {RQ: {}},  # when bound direct to a 13:
-    # },
-    # DEV_KLASS.DTS: {  # e.g. DTS92(E)
-    #     _0001: {W_: {}},
-    #     _0008: {I_: {}},
-    #     _0009: {I_: {}},
-    #     _000A: {I_: {}, RQ: {}, W_: {}},
-    #     _0016: {RQ: {}},
-    #     # "0B04": {I_: {}},
-    #     _1030: {I_: {}},
-    #     _1060: {I_: {}},
-    #     _1090: {RQ: {}},
-    #     _1100: {I_: {}},
-    #     _1F09: {I_: {}},
-    #     _1FC9: {I_: {}},
-    #     _2309: {I_: {}, RQ: {}, W_: {}},
-    #     _2349: {RQ: {}, W_: {}},
-    #     _30C9: {I_: {}},
-    #     _313F: {I_: {}},
-    #     _3B00: {I_: {}},
-    #     _3EF1: {RQ: {}},
-    # },
-    # DEV_KLASS.HCW: {  # e.g. HCF82/HCW82: Room Temperature Sensor
-    #     _0001: {W_: {}},
-    #     _0002: {I_: {}},
-    #     _0008: {I_: {}},
-    #     _0009: {I_: {}},
-    #     _1060: {I_: {}},
-    #     _1100: {I_: {}},
-    #     _1F09: {I_: {}},
-    #     _1FC9: {I_: {}},
-    #     _2309: {I_: {}},
-    #     _2389: {I_: {}},
-    #     _30C9: {I_: {}},
-    # },
     DEV_KLASS.UFC: {  # e.g. HCE80/HCC80: Underfloor Heating Controller
         _0001: {RP: {}, W_: {}},  # TODO: Ix RP
         _0005: {RP: {}},
@@ -937,10 +885,11 @@ RAMSES_DEVICES_CLASS = {
         # RP: {},  # RQ --- 01:145038 13:237335 --:------ 3EF0 001 00
         _3EF1: {RP: {}},
     },
-    DEV_KLASS.EXT: {
+    DEV_KLASS.OUT: {
         _0002: {I_: {}},
         _1FC9: {I_: {}},
     },  # i.e. HB85 (ext. temperature/luminosity(lux)), HB95 (+ wind speed)
+    #
     DEV_KLASS.JIM: {  # Jasper Interface Module, 08
         _0008: {RQ: {}},
         _10E0: {I_: {}},
@@ -953,6 +902,58 @@ RAMSES_DEVICES_CLASS = {
         _10E0: {I_: {}},
         _3EF1: {RQ: {}, RP: {}},
     },
+    # DEV_KLASS.RND: {  # e.g. TR87RF: Single (round) Zone Thermostat
+    #     _0005: {I_: {}},
+    #     _0008: {I_: {}},
+    #     _000A: {I_: {}, RQ: {}},
+    #     _000C: {I_: {}},
+    #     _000E: {I_: {}},
+    #     _042F: {I_: {}},
+    #     _1060: {I_: {}},
+    #     _10E0: {I_: {}},
+    #     _12C0: {I_: {}},
+    #     _1FC9: {I_: {}},
+    #     _2309: {I_: {}, RQ: {}, W_: {}},
+    #     _2349: {RQ: {}},
+    #     _30C9: {I_: {}},
+    #     _3120: {I_: {}},
+    #     _313F: {I_: {}},  # W --- 30:253184 34:010943 --:------ 313F 009 006000070E0...
+    #     _3EF0: {RQ: {}},  # when bound direct to a 13:
+    #     _3EF1: {RQ: {}},  # when bound direct to a 13:
+    # },
+    # DEV_KLASS.DTS: {  # e.g. DTS92(E)
+    #     _0001: {W_: {}},
+    #     _0008: {I_: {}},
+    #     _0009: {I_: {}},
+    #     _000A: {I_: {}, RQ: {}, W_: {}},
+    #     _0016: {RQ: {}},
+    #     # "0B04": {I_: {}},
+    #     _1030: {I_: {}},
+    #     _1060: {I_: {}},
+    #     _1090: {RQ: {}},
+    #     _1100: {I_: {}},
+    #     _1F09: {I_: {}},
+    #     _1FC9: {I_: {}},
+    #     _2309: {I_: {}, RQ: {}, W_: {}},
+    #     _2349: {RQ: {}, W_: {}},
+    #     _30C9: {I_: {}},
+    #     _313F: {I_: {}},
+    #     _3B00: {I_: {}},
+    #     _3EF1: {RQ: {}},
+    # },
+    # DEV_KLASS.HCW: {  # e.g. HCF82/HCW82: Room Temperature Sensor
+    #     _0001: {W_: {}},
+    #     _0002: {I_: {}},
+    #     _0008: {I_: {}},
+    #     _0009: {I_: {}},
+    #     _1060: {I_: {}},
+    #     _1100: {I_: {}},
+    #     _1F09: {I_: {}},
+    #     _1FC9: {I_: {}},
+    #     _2309: {I_: {}},
+    #     _2389: {I_: {}},
+    #     _30C9: {I_: {}},
+    # },
 }
 
 HVAC_DEVICES_CLASS = {
@@ -978,6 +979,7 @@ HVAC_DEVICES_CLASS = {
         _31E0: {I_: {}},
     },
     DEV_KLASS.HUM: {
+        _042F: {I_: {}},
         _1060: {I_: {}},
         _10E0: {I_: {}},
         _12A0: {I_: {}},
