@@ -4,6 +4,15 @@
 """RAMSES RF - RAMSES-II compatible Packet processor.
 
 Operates at the pkt layer of: app - msg - pkt - h/w
+
+For ser2net, use the following YAML file with: ser2net -c hgi80.yaml
+
+connection: &con00
+  accepter: telnet(rfc2217),tcp,5001
+  timeout: 0
+  connector: serialdev,/dev/ttyUSB0,115200n81,local
+  options:
+    max-connections: 3
 """
 
 import asyncio
@@ -18,16 +27,15 @@ from queue import Queue
 from string import printable  # ascii_letters, digits
 from threading import Lock, Thread
 from types import SimpleNamespace
-from typing import ByteString, Callable, Generator, Optional, Tuple
+from typing import ByteString, Callable, Iterable, Optional, Tuple
 
 from serial import SerialException, serial_for_url
 from serial_asyncio import SerialTransport as SerTransportAsync
 
-from .command import (
+from .command import (  # QOS_RX_TIMEOUT,
     ARGS,
     DEAMON,
     FUNC,
-    QOS_RX_TIMEOUT,
     QOS_TX_RETRIES,
     QOS_TX_TIMEOUT,
     Command,
@@ -63,6 +71,9 @@ POLLER_TASK = "poller_task"
 _QOS_POLL_INTERVAL = 0.005
 _QOS_MAX_BACKOFF = 3
 
+_MIN_GAP_BETWEEN_WRITES = 0.2  # seconds
+_MIN_GAP_BETWEEN_RETRYS = td(seconds=2.0)  # seconds
+
 Pause = SimpleNamespace(
     NONE=td(seconds=0),
     MINIMUM=td(seconds=0.01),
@@ -291,11 +302,10 @@ class PacketProtocolBase(asyncio.Protocol):
     to transport: self.send_data(cmd)       -> self._transport.write(bytes)
     """
 
-    @staticmethod
-    def _dt_now():
-        return dt_now() if sys.platform == "win32" else dt.now()
-
     def __init__(self, gwy, pkt_handler: Callable) -> None:
+
+        _LOGGER.info(f"RAMSES_RF protocol library v{VERSION}, using {self}")
+
         self._gwy = gwy
         self._loop = gwy._loop
         self._callback = pkt_handler  # Could be None
@@ -319,15 +329,27 @@ class PacketProtocolBase(asyncio.Protocol):
             self._include = []
         self._unwanted = []  # not: [NON_DEVICE_ID, NUL_DEVICE_ID]
 
+        self._dt_now_ = dt_now if sys.platform == "win32" else dt.now
+
         self._hgi80 = {
             IS_INITIALIZED: None,
             IS_EVOFW3: None,
             DEVICE_ID: None,
         }
 
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}(enforce_include={self.enforce_include})"
+
+    def __str__(self) -> str:
+        return self.__class__.__name__
+
+    def _dt_now(self) -> dt:
+        """Return a precise datetime, using the curent dtm."""
+        return self._dt_now_()  # or, simply: return dt_now()
+
     def connection_made(self, transport: asyncio.Transport) -> None:
         """Called when a connection is made."""
-        # _LOGGER.debug("PktProtocol.connection_made(%s)", transport)
+        _LOGGER.debug(f"{self}.connection_made({transport})")
 
         self._transport = transport
 
@@ -345,8 +367,118 @@ class PacketProtocolBase(asyncio.Protocol):
                 "is strongly recommended)"
             )
 
-    # @functools.lru_cache(maxsize=128)  # this will no longer work
-    def _is_wanted(self, src_id, dst_id) -> Optional[bool]:
+    def connection_lost(self, exc: Optional[Exception]) -> None:
+        """Called when the connection is lost or closed."""
+        _LOGGER.debug(f"{self}.connection_lost(exc)")
+        # serial.serialutil.SerialException: device reports error (poll)
+
+        if exc is not None:
+            raise exc
+
+    def pause_writing(self) -> None:
+        """Called when the transport's buffer goes over the high-water mark."""
+        _LOGGER.debug(f"{self}.pause_writing()")
+
+        self._pause_writing = True
+
+    def resume_writing(self) -> None:
+        """Called when the transport's buffer drains below the low-water mark."""
+        _LOGGER.debug(f"{self}.resume_writing()")
+
+        self._pause_writing = False
+
+    def data_received(self, data: ByteString) -> None:
+        """Called by the transport when some data (packet fragments) is received."""
+        # _LOGGER.debug(f"{self}.data_received({%s})", data)
+
+        def bytes_received(data: ByteString) -> Iterable[ByteString]:
+            self._recv_buffer += data
+            if b"\r\n" in self._recv_buffer:
+                lines = self._recv_buffer.split(b"\r\n")
+                self._recv_buffer = lines[-1]
+                for line in lines[:-1]:
+                    yield self._dt_now(), line
+
+        for dtm, raw_line in bytes_received(data):
+            self._line_received(dtm, _normalise(_str(raw_line)), raw_line)
+
+    def _line_received(self, dtm: dt, line: str, raw_line: ByteString) -> None:
+
+        if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
+            _LOGGER.info("RF Rx: %s", raw_line)
+
+        self._hgi80[IS_INITIALIZED], was_initialized = True, self._hgi80[IS_INITIALIZED]
+
+        try:
+            pkt = Packet.from_port(
+                self._gwy,
+                dtm,
+                _regex_hack(line, self._gwy.config.use_regex.get("inbound", {})),
+                raw_line=raw_line,
+            )  # should log all? invalid pkts appropriately
+
+            if pkt.src.type == "18":  # dex: ideally should use HGI, but how?
+                if self._hgi80[DEVICE_ID] is None:
+                    self._hgi80[DEVICE_ID] = pkt.src.id
+
+                elif self._hgi80[DEVICE_ID] != pkt.src.id:
+                    (
+                        _LOGGER.debug
+                        if pkt.src.id in self._unwanted
+                        else _LOGGER.warning
+                    )(
+                        f"{pkt} < There appears to be more than one HGI80-compatible device"
+                        f" (active gateway: {self._hgi80[DEVICE_ID]}), this is unsupported"
+                    )
+
+        except InvalidPacketError as exc:
+            if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
+                self._hgi80[IS_EVOFW3] = line
+                if self._evofw_flag not in (None, "!V"):
+                    self._transport.write(
+                        bytes(f"{self._evofw_flag}\r\n".encode("ascii"))
+                    )
+            elif was_initialized and line and line[:1] != "#" and "*" not in line:
+                _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
+            return
+
+        try:
+            self._pkt_received(pkt)
+
+        except (  # protect the code that invoked the callback from this layer
+            ArithmeticError,  # incl. ZeroDivisionError,
+            AssertionError,
+            AttributeError,
+            LookupError,  # incl. IndexError, KeyError
+            NameError,  # incl. UnboundLocalError
+            RuntimeError,  # incl. RecursionError
+            TypeError,
+            ValueError,
+        ) as exc:  # noqa: E722, broad-except
+            _LOGGER.exception("%s < exception from pkt layer: %s", pkt, exc)
+
+    def _pkt_received(self, pkt: Packet) -> None:
+        """Pass any valid/wanted packets to the callback."""
+
+        self._this_pkt, self._prev_pkt = pkt, self._this_pkt
+
+        if self._callback and self._is_wanted(pkt.src.id, pkt.dst.id):
+            try:
+                self._callback(pkt)  # only wanted PKTs to the MSG transport's handler
+
+            except (  # protect this code from the upper-layer callback
+                ArithmeticError,  # incl. ZeroDivisionError,
+                AssertionError,
+                AttributeError,
+                LookupError,  # incl. IndexError, KeyError
+                NameError,  # incl. UnboundLocalError
+                RuntimeError,  # incl. RecursionError
+                TypeError,
+                ValueError,
+            ) as exc:  # noqa: E722, broad-except
+                _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
+
+    def _is_wanted(self, src_id: str, dst_id: str) -> Optional[bool]:
         """Parse the packet, return True if the packet is not to be filtered out.
 
         An unwanted device_id will 'trump' a whitelited device_id in the same packet
@@ -388,95 +520,137 @@ class PacketProtocolBase(asyncio.Protocol):
                 self._unwanted.append(dev_id)
                 return
 
-            if self.enforce_include:  # self._include:
-                (_LOGGER.info if dev_id in self._exclude else _LOGGER.warning)(
-                    f"Blocking packets with device_id: {dev_id} (is not whitelisted), "
-                    f"if required, add it to the {KNOWN_LIST}"
-                )
+            if self.enforce_include:
+                if self._exclude:
+                    _LOGGER.info(
+                        f"Blocking packets with device_id: {dev_id} (not blacklisted), "
+                        f"if required, add it to the {KNOWN_LIST}"
+                    )
+                else:  # and not self._exclude:
+                    _LOGGER.warning(
+                        f"Blocking packets with device_id: {dev_id} (no blacklist), "
+                        f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
+                    )
                 self._unwanted.append(dev_id)
                 return
 
         return True
 
-    def _pkt_received(self, pkt: Packet) -> None:
-        """Pass any valid/wanted packets to the callback."""
+    async def send_data(self, cmd: Command) -> None:
+        raise NotImplementedError
 
-        self._this_pkt, self._prev_pkt = pkt, self._this_pkt
 
-        if self._callback and self._is_wanted(pkt.src.id, pkt.dst.id):
-            try:
-                self._callback(pkt)  # only wanted PKTs to the MSG transport's handler
-            except InvalidPacketError as exc:
-                _LOGGER.error("%s < %s", pkt, exc)
-            # except Exception as exc:  # noqa: E722, broad-except
-            except (
-                ArithmeticError,  # incl. ZeroDivisionError,
-                AssertionError,
-                AttributeError,
-                IndexError,
-                LookupError,  # incl. IndexError, KeyError
-                NameError,  # incl. UnboundLocalError
-                RuntimeError,  # incl. RecursionError
-                TypeError,
-                ValueError,
-            ) as exc:
-                _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
+class PacketProtocolFile(PacketProtocolBase):
+    """Interface for a packet protocol (for packet log)."""
 
-    def _line_received(self, dtm: dt, line: str, raw_line: ByteString) -> None:
-        if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
-            _LOGGER.info("RF Rx: %s", raw_line)
+    def __init__(self, gwy, pkt_handler: Callable) -> None:
+        super().__init__(gwy, pkt_handler)
 
-        self._hgi80[IS_INITIALIZED], was_initialized = True, self._hgi80[IS_INITIALIZED]
+        self._dt_str_ = None
+
+    def _dt_now(self) -> dt:
+        """Return a precise datetime, using a packet's dtm field."""
 
         try:
-            pkt = Packet.from_port(
+            return dt.fromisoformat(self._dt_str_)  # always current pkt's dtm
+        except (TypeError, ValueError):
+            pass
+
+        try:
+            return self._this_pkt.dtm  # if above fails, will be previous pkt's dtm
+        except AttributeError:
+            return dt(1970, 1, 1, 1, 0)
+
+    def data_received(self, data: str) -> None:
+        """Called when a packet line is received (from a log file)."""
+        # _LOGGER.debug(f"{self}.data_received(%s)", data.rstrip())
+
+        self._dt_str_ = data[:26]  # used for self._dt_now
+
+        self._line_received(data[:26], data[27:].strip(), data)
+
+    def _line_received(self, dtm: str, line: str, raw_line: str) -> None:
+
+        try:
+            pkt = Packet.from_file(
                 self._gwy,
                 dtm,
                 _regex_hack(line, self._gwy.config.use_regex.get("inbound", {})),
-                raw_line=raw_line,
-            )
+            )  # should log all invalid pkts appropriately
 
-        except InvalidPacketError as exc:
-            if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
-                self._hgi80[IS_EVOFW3] = line
-                if self._evofw_flag not in (None, "!V"):
-                    self._transport.write(
-                        bytes(f"{self._evofw_flag}\r\n".encode("ascii"))
-                    )
-            elif was_initialized and line and line[:1] != "#" and "*" not in line:
-                _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
+        except (InvalidPacketError, ValueError):  # VE from dt.fromisoformat()
             return
 
-        if pkt.src.type == "18":  # dex: ideally should use HGI, but how?
+        if pkt.src.type == "18" and pkt.src.id != HGI_DEVICE_ID:  # HACK 01: dex
             if self._hgi80[DEVICE_ID] is None:
                 self._hgi80[DEVICE_ID] = pkt.src.id
 
             elif self._hgi80[DEVICE_ID] != pkt.src.id:
-                (_LOGGER.debug if pkt.src.id in self._unwanted else _LOGGER.warning)(
+                _LOGGER.debug(
                     f"{pkt} < There appears to be more than one HGI80-compatible device"
                     f" (active gateway: {self._hgi80[DEVICE_ID]}), this is unsupported"
                 )
 
         self._pkt_received(pkt)
 
-    def data_received(self, data: ByteString) -> None:
-        """Called when some data (packet fragments) is received (from RF)."""
-        # _LOGGER.debug("PacketProtocolBase.data_received(%s)", data.rstrip())
 
-        def _bytes_received(
-            data: ByteString,
-        ) -> Generator[ByteString, ByteString, None]:
-            self._recv_buffer += data
-            if b"\r\n" in self._recv_buffer:
-                lines = self._recv_buffer.split(b"\r\n")
-                self._recv_buffer = lines[-1]
-                for line in lines[:-1]:
-                    yield self._dt_now(), line
+class PacketProtocolPort(PacketProtocolBase):
+    """Interface for a packet protocol (without QoS)."""
 
-        for dtm, raw_line in _bytes_received(data):
-            self._line_received(dtm, _normalise(_str(raw_line)), raw_line)
+    def __init__(self, gwy, pkt_handler: Callable) -> None:
+        super().__init__(gwy, pkt_handler)
 
-    async def _send_data(self, data: str) -> None:
+        self._sem = asyncio.BoundedSemaphore()
+        self._loop.create_task(self._leak_sem())
+
+    async def _leak_sem(self):
+        """Used to enforce a minimum time between calls to self._transport.write()."""
+        while True:
+            await asyncio.sleep(_MIN_GAP_BETWEEN_WRITES)
+            try:
+                self._sem.release()
+            except ValueError:
+                pass
+
+    def connection_made(self, transport: asyncio.Transport) -> None:
+        """Called when a connection is made."""
+
+        super().connection_made(transport)  # self._transport = transport
+        # self._transport.serial.rts = False
+
+        # determine if using a evofw3 rather than a HGI80
+        self._transport.write(bytes("!V\r\n".encode("ascii")))
+
+        # add this to start of the pkt log, if any
+        if not self._disable_sending:  # TODO: use a callback
+            self._loop.create_task(self.send_data(Command._puzzle()))
+
+        self.resume_writing()
+
+    async def send_data(self, cmd: Command) -> None:
+        """Called when some data is to be sent (not a callback)."""
+        # _LOGGER.debug(f"{self}.send_data(%s)", cmd)
+
+        if self._disable_sending or self._pause_writing:
+            raise RuntimeError("Sending is disabled or writing is paused")
+
+        if cmd.src.id != HGI_DEVICE_ID:
+            if self._hgi80[IS_EVOFW3]:
+                _LOGGER.info(
+                    "Impersonating device: %s, for pkt: %s", cmd.src.id, cmd.tx_header
+                )
+            else:
+                _LOGGER.warning(
+                    "Impersonating device: %s, for pkt: %s"
+                    ", NB: standard HGI80s dont support this feature, it needs evofw3!",
+                    cmd.src.id,
+                    cmd.tx_header,
+                )
+            await self.send_data(Command._puzzle(msg_type="11", message=cmd.tx_header))
+
+        await self._send_data(str(cmd))
+
+    async def _send_data(self, data: str) -> None:  # throtled
         """Send a bytearray to the transport (serial) interface."""
 
         while self._pause_writing:
@@ -496,146 +670,17 @@ class PacketProtocolBase(asyncio.Protocol):
             ).encode("ascii")
         )
 
+        await self._sem.acquire()  # try not to exceed a (reasonable) RF duty cycle
+
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
             _LOGGER.info("RF Tx:     %s", data)
         self._transport.write(data + b"\r\n")
 
-        # 0.2: can still exceed RF duty cycle limit with back-to-back restarts
-        # await asyncio.sleep(0.2)  # TODO: RF Duty cycle, make configurable?
-
-    async def send_data(self, cmd: Command) -> None:
-        """Called when some data is to be sent (not a callback)."""
-        # _LOGGER.debug("PktProtocol.send_data(%s)", cmd)
-
-        if self._disable_sending:
-            raise RuntimeError("Sending is disabled")
-
-        if cmd.src.id != HGI_DEVICE_ID:
-            if self._hgi80[IS_EVOFW3]:
-                _LOGGER.info(
-                    "Impersonating device: %s, for pkt: %s", cmd.src.id, cmd.tx_header
-                )
-            else:
-                _LOGGER.warning(
-                    "Impersonating device: %s, for pkt: %s"
-                    ", NB: standard HGI80s dont support this feature, it needs evofw3!",
-                    cmd.src.id,
-                    cmd.tx_header,
-                )
-            await self.send_data(Command._puzzle(msg_type="11", message=cmd.tx_header))
-
-        await self._send_data(str(cmd))
-
-    def connection_lost(self, exc: Optional[Exception]) -> None:
-        """Called when the connection is lost or closed."""
-        _LOGGER.debug("PktProtocol.connection_lost(%s)", exc)
-        # serial.serialutil.SerialException: device reports error (poll)
-
-        if exc is not None:
-            raise exc
-
-    def pause_writing(self) -> None:
-        """Called when the transport's buffer goes over the high-water mark."""
-        _LOGGER.debug("PktProtocol.pause_writing()")
-        self._pause_writing = True
-
-    def resume_writing(self) -> None:
-        """Called when the transport's buffer drains below the low-water mark."""
-        _LOGGER.debug("PktProtocol.resume_writing()")
-        self._pause_writing = False
-
-
-class PacketProtocolPort(PacketProtocolBase):
-    """Interface for a packet protocol (without QoS)."""
-
-    def connection_made(self, transport: asyncio.Transport) -> None:
-        """Called when a connection is made."""
-        _LOGGER.info(f"RAMSES_RF protocol library v{VERSION} (serial port)")
-
-        super().connection_made(transport)  # self._transport = transport
-        # self._transport.serial.rts = False
-
-        # determine if using a evofw3 rather than a HGI80
-        self._transport.write(bytes("!V\r\n".encode("ascii")))
-
-        # add this to start of the pkt log, if any
-        if not self._disable_sending:  # TODO: use a callback
-            self._loop.create_task(self.send_data(Command._puzzle()))
-
-        self.resume_writing()
-
-
-class PacketProtocol(PacketProtocolPort):
-    """Interface for a packet protocol (without QoS)."""
-
-    def __init__(self, gwy, pkt_handler: Callable) -> None:
-        _LOGGER.debug(
-            "PktProtocol.__init__(gwy, %s) *** Std version ***",
-            pkt_handler.__name__ if pkt_handler else None,
-        )
-        super().__init__(gwy, pkt_handler)
-
-
-class PacketProtocolRead(PacketProtocolBase):
-    """Interface for a packet protocol (for packet log)."""
-
-    def __init__(self, gwy, pkt_handler: Callable) -> None:
-        _LOGGER.debug(
-            "PacketProtocolRead.__init__(gwy, %s) *** R/O version ***",
-            pkt_handler.__name__ if pkt_handler else None,
-        )
-        super().__init__(gwy, pkt_handler)
-
-    def connection_made(self, transport: asyncio.Transport) -> None:
-        """Called when a connection is made."""
-        _LOGGER.info(f"RAMSES_RF protocol library v{VERSION} (packet log)")
-
-        super().connection_made(transport)  # self._transport = transport
-
-    def _line_received(self, dtm: str, line: str, raw_line: str) -> None:
-
-        try:
-            pkt = Packet.from_file(
-                self._gwy,
-                dtm,
-                _regex_hack(line, self._gwy.config.use_regex.get("inbound", {})),
-            )
-
-        except (InvalidPacketError, ValueError):  # VE from dt.fromisoformat()
-            return
-
-        if pkt.src.type == "18" and pkt.src.id != HGI_DEVICE_ID:  # HACK 01: dex
-            if self._hgi80[DEVICE_ID] is None:
-                self._hgi80[DEVICE_ID] = pkt.src.id
-
-            elif self._hgi80[DEVICE_ID] != pkt.src.id:
-                _LOGGER.debug(
-                    f"{pkt} < There appears to be more than one HGI80-compatible device"
-                    f" (active gateway: {self._hgi80[DEVICE_ID]}), this is unsupported"
-                )
-
-        self._pkt_received(pkt)
-
-    def data_received(self, data: str) -> None:
-        """Called when a packet line is received (from a log file)."""
-        # _LOGGER.debug("PacketProtocolRead.data_received(%s)", data.rstrip())
-        self._line_received(data[:26], data[27:].strip(), data)
-
-    def _dt_now(self) -> dt:
-        try:
-            return self._this_pkt.dtm
-        except AttributeError:
-            return dt(1970, 1, 1, 1, 0)
-
 
 class PacketProtocolQos(PacketProtocolPort):
     """Interface for a packet protocol (includes QoS)."""
 
     def __init__(self, gwy, pkt_handler: Callable) -> None:
-        _LOGGER.debug(
-            "PktProtocol.__init__(gwy, %s) *** Qos version ***",
-            pkt_handler.__name__ if pkt_handler else None,
-        )
         super().__init__(gwy, pkt_handler)
 
         self._qos_lock = Lock()
@@ -649,52 +694,6 @@ class PacketProtocolQos(PacketProtocolPort):
         self._timeout_full = None
         self._timeout_half = None
 
-    def _qos_set_cmd(self, cmd) -> None:
-        """Set the QoS command for sending, or clear it when sent OK/timed out."""
-
-        self._qos_lock.acquire()
-        if cmd and self._qos_cmd:
-            raise RuntimeError
-        self._qos_cmd = cmd
-        self._qos_lock.release()
-
-        self._tx_hdr = cmd.tx_header if cmd else None
-        self._rx_hdr = cmd.rx_header if cmd else None  # Could be None, even if cmd
-
-        if cmd:
-            self._tx_retries = 0
-            self._tx_retry_limit = cmd.qos.get("retries", QOS_TX_RETRIES)
-
-    def _qos_expire_cmd(self, cmd) -> None:
-        """Handle an expired cmd, such as invoking its callbacks."""
-
-        if cmd._source_entity:  # HACK - should be using a callback
-            cmd._source_entity._qos_function(cmd)
-
-        hdr, callback = cmd.tx_header, cmd.callback
-        if callback and not callback.get("expired"):
-            # see also: MsgTransport._pkt_receiver()
-            _LOGGER.error("PktProtocolQos.send_data(%s): Expired callback", hdr)
-            callback[FUNC](False, *callback.get(ARGS, tuple()))
-            callback["expired"] = not callback.get(DEAMON, False)  # HACK:
-
-    def _qos_update_timeouts(self) -> None:
-        """Update QoS self._timeout_full, self._timeout_half attrs."""
-
-        if not self._qos_cmd:
-            raise RuntimeError
-
-        dtm = self._dt_now()
-
-        if self._tx_hdr:
-            timeout = QOS_TX_TIMEOUT
-        else:
-            timeout = self._qos_cmd.qos.get("timeout", QOS_RX_TIMEOUT)
-        timeout = min(timeout * 4 ** self._backoff, td(seconds=1))
-
-        self._timeout_full = dtm + timeout * 2
-        self._timeout_half = dtm + timeout
-
     def _pkt_received(self, pkt: Packet) -> None:
         """Called when packets are received (a callback).
 
@@ -762,11 +761,8 @@ class PacketProtocolQos(PacketProtocolPort):
         logger_rcvd(msg, wanted=wanted)
 
     async def send_data(self, cmd: Command) -> None:
-        """Called when packets are to be sent (not a callback).
-
-        Wraps the relevant function with QoS code.
-        """
-        # _LOGGER.debug("PktProtocolQos.send_data(%s)", cmd)
+        """Called when packets are to be sent (not a callback)."""
+        # _LOGGER.debug(f"{self}.send_data(%s)", cmd)
 
         while self._qos_cmd is not None:
             await asyncio.sleep(_QOS_POLL_INTERVAL)
@@ -820,10 +816,64 @@ class PacketProtocolQos(PacketProtocolPort):
             if self._timeout_half >= self._dt_now():
                 self._backoff = max(self._backoff - 1, 0)
 
+    def _qos_set_cmd(self, cmd) -> None:
+        """Set the QoS command for sending, or clear it when sent OK/timed out."""
+
+        self._qos_lock.acquire()
+        if cmd and self._qos_cmd:
+            raise RuntimeError
+        self._qos_cmd = cmd
+        self._qos_lock.release()
+
+        self._tx_hdr = cmd.tx_header if cmd else None
+        self._rx_hdr = cmd.rx_header if cmd else None  # Could be None, even if cmd
+
+        if cmd:
+            self._tx_retries = 0
+            self._tx_retry_limit = cmd.qos.get("retries", QOS_TX_RETRIES)
+
+    def _qos_update_timeouts(self) -> None:
+        """Update QoS self._timeout_full, self._timeout_half attrs."""
+
+        if not self._qos_cmd:
+            raise RuntimeError
+
+        dtm = self._dt_now()
+
+        if self._tx_hdr:
+            timeout = QOS_TX_TIMEOUT  # td(seconds=0.05)
+        else:
+            # timeout = self._qos_cmd.qos.get("timeout", QOS_RX_TIMEOUT)
+            timeout = _MIN_GAP_BETWEEN_RETRYS  # td(seconds=2.0)
+
+        # timeout = min(timeout * 4 ** self._backoff, td(seconds=1))
+        timeout = min(timeout * 4**self._backoff, _MIN_GAP_BETWEEN_RETRYS)
+
+        self._timeout_full = dtm + timeout
+        self._timeout_half = dtm + timeout / 2
+
+        # self._timeout_full = dtm + _MIN_GAP_BETWEEN_RETRYS  # was: + timeout * 2
+        # self._timeout_half = dtm + _MIN_GAP_BETWEEN_RETRYS  # was: + timeout
+
+    def _qos_expire_cmd(self, cmd) -> None:
+        """Handle an expired cmd, such as invoking its callbacks."""
+
+        if cmd._source_entity:  # HACK - should be using a callback
+            cmd._source_entity._qos_function(cmd)
+
+        hdr, callback = cmd.tx_header, cmd.callback
+        if callback and not callback.get("expired"):
+            # see also: MsgTransport._pkt_receiver()
+            _LOGGER.error("PktProtocolQos.send_data(%s): Expired callback", hdr)
+            callback[FUNC](False, *callback.get(ARGS, tuple()))
+            callback["expired"] = not callback.get(DEAMON, False)  # HACK:
+
 
 def create_pkt_stack(
     gwy,
     pkt_callback,
+    /,
+    *,
     protocol_factory=None,
     ser_port=None,
     packet_log=None,
@@ -831,7 +881,7 @@ def create_pkt_stack(
 ) -> Tuple[asyncio.Protocol, asyncio.Transport]:
     """Utility function to provide a transport to the internal protocol.
 
-    The architecture is: app (client) -> msg -> pkt -> ser (HW interface).
+    The architecture is: app (client) -> msg -> pkt -> ser (HW interface) / log (file).
 
     The msg/pkt interface is via:
      - PktProtocol.data_received           to (pkt_callback) MsgTransport._pkt_receiver
@@ -849,28 +899,26 @@ def create_pkt_stack(
             "(e.g. linux with a local serial port)"
         )
 
-    def _protocol_factory():
+    def protocol_factory_():
         if packet_log or packet_dict is not None:
-            return create_protocol_factory(PacketProtocolRead, gwy, pkt_callback)()
-        elif gwy.config.disable_sending:  # TODO: assumes we wont change our mind
-            return create_protocol_factory(PacketProtocol, gwy, pkt_callback)()
+            return create_protocol_factory(PacketProtocolFile, gwy, pkt_callback)()
+        elif gwy.config.disable_sending:  # NOTE: assumes we wont change our mind
+            return create_protocol_factory(PacketProtocolPort, gwy, pkt_callback)()
         else:
             return create_protocol_factory(PacketProtocolQos, gwy, pkt_callback)()
 
     if len([x for x in (packet_dict, packet_log, ser_port) if x is not None]) != 1:
-        raise TypeError("port / file / dict should be mutually exclusive")
+        raise TypeError("serial port, log file & dict should be mutually exclusive")
 
-    pkt_protocol = (protocol_factory or _protocol_factory)()
+    pkt_protocol = (protocol_factory or protocol_factory_)()
 
-    if packet_log or packet_dict is not None:  # {} is a processable packet_dict
-        pkt_transport = SerTransportRead(
-            gwy._loop, pkt_protocol, packet_log or packet_dict
-        )
+    if (log_file := packet_log or packet_dict) is not None:  # {} is a processable log
+        pkt_transport = SerTransportRead(gwy._loop, pkt_protocol, log_file)
         return (pkt_protocol, pkt_transport)
 
-    ser_config = DEFAULT_SERIAL_CONFIG
-    ser_config.update(gwy.config.serial_config)
+    ser_config = {**DEFAULT_SERIAL_CONFIG, **gwy.config.serial_config}
 
+    # python client.py monitor 'rfc2217://localhost:5001'
     # python client.py monitor 'alt:///dev/ttyUSB0?class=PosixPollSerial'
     try:
         ser_instance = serial_for_url(ser_port, **ser_config)
@@ -883,16 +931,11 @@ def create_pkt_stack(
     except (AttributeError, NotImplementedError, ValueError):  # Wrong OS/Platform/FTDI
         pass
 
-    if any(
-        (
-            ser_port.startswith("rfc2217:"),
-            ser_port.startswith("socket:"),
-            os.name == "nt",
-        )
-    ):
+    # TODO: remove SerTransportPoll hack
+    if os.name == "nt" or ser_port[:8] == "rfc2217:" or ser_port[:7] == "socket:":
         issue_warning()
         pkt_transport = SerTransportPoll(gwy._loop, pkt_protocol, ser_instance)
-    else:
+    else:  # use the standard serial_asyncio library
         pkt_transport = SerTransportAsync(gwy._loop, pkt_protocol, ser_instance)
 
     return (pkt_protocol, pkt_transport)
